/*
 *  Copyright (c) 1996-2007 Digi International Inc., All Rights Reserved
 *
 *  This software contains proprietary and confidential information of Digi
 *  International Inc.  By accepting transfer of this copy, Recipient agrees
 *  to retain this software in confidence, to prevent disclosure to others,
 *  and to make no use of this software other than that for which it was
 *  delivered.  This is an unpublished copyrighted work of Digi International
 *  Inc.  Except as permitted by federal law, 17 USC 117, copying is strictly
 *  prohibited.
 *
 *  Restricted Rights Legend
 *
 *  Use, duplication, or disclosure by the Government is subject to
 *  restrictions set forth in sub-paragraph (c)(1)(ii) of The Rights in
 *  Technical Data and Computer Software clause at DFARS 252.227-7031 or
 *  subparagraphs (c)(1) and (2) of the Commercial Computer Software -
 *  Restricted Rights at 48 CFR 52.227-19, as applicable.
 *
 *  Digi International Inc. 11001 Bren Road East, Minnetonka, MN 55343
 *
 * =======================================================================
 *    Declarations for naflash.c
 *
 * Edit Date/Ver   Edit Description
 * ==============  =======================================================
 * LZ   11/05/99   Ported to NETOS from VX_WORKS.
 * LZ   12/21/99   Rewrite some portion of the code to support 16-bit and 32-bit mode flash.
 * LZ   12/21/99   Added "total_sector_number" field to struct "flash_id_table_t"
 * LZ   04/19/00   Changed the return error code and added more return code.
 * LZ   05/01/00   Added support to FUJITSU flash.
 * JF   06/30/00   Added support to ATMEL flash
 * JF	06/30/00   Added SST39VF800 support.
 * JF    9/11/00   Added support for STM_M29W800AB
 * DXL	03/08/02   Added support for ATMEL AT49BV1614A
 * JW   03/28/02   Added new defines to support new API functions
 * JW   04/05/02   Added new defines (with NAFLASH_ prefix) for return values
 * JW   04/08/02   Added new define (MAX_SECTORS) to support new implementation of LSetSectorAddress()
 * CHG  04/09/02   Added support for S29W320DB.
 * JW   04/16/02   1) Added new define (MAX_SECTOR_SIZE) to specify size of first_sector_data[] buffer
 *                 2) Changed flash sectors for AM29DL323D from 0x20 to 0x47
 * JW   04/27/02   Added new API function prototypes
 * JW   05/01/02   1) Added new #define (MAX_FLASH_BANKS) to limit the number of flash banks supported.
 *                 2) Added new #define (NAFLASH_DUPLICATE_FLASHBANKS) for error codes
 *                 3) Moved the API prototypes to naflash.h
 * JW   05/03/02   1) Changed NAFLASH_DUPLICATE_FLASHBANKS to NAFLASH_DUPLICATE_CALL
 *                 2) Added new #define (NAFLASH_SEMAPHORE_CREATE_FAILED)
 * JW   05/06/02   Included Npttypes.h
 * JW   04/29/03   Changed MAX_SECTORS from 128 to 142 to support two banks of 4MB flash parts that have
 *                 variable sector sizes (i.e. AMD29DL323).
 * LO  06/05/03		Added Flash support for AMD AM29LV160 Top/Bottom
 *					Added Flash support for Fujitsu MBM29LV160 Top/Bottom
 *					Provided Flash support for STM M29LV160 Top/Bottom not tested
 *					Added Flash support for AMD AM29LV641
 * HB	27/4/04	   Added Fujitsu MBM29LV650U & Macronix MX29LV640BUTI90 (AMD AM29LV641 Compatible) 
 * HB 	27/4/04    Added ST Microelectronic M29W641DL and M29W641DH (AMD AM29LV641 Compatible) 
 * HB 	27/4/04    Added AMD MirrorBit Tech AM29LV641MH/L (AMD AM29LV641 Compatible) 
 * HB   18/5/04	   Changed struct flash_id_table_t to support second ID code
 *
 */

#ifndef FLASH_H
#define FLASH_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "Npttypes.h"

/* Sector Information for each flash chip */

/* Spansion S29GL064N */
#define SPANSION_S29GL064N_FLASH_SECTORS        0x080U
#define SPANSION_S29GL064N_FLASH_SECTOR_SIZE    0x10000
#define SPANSION_S29GL064N_PROG_SECTOR_SIZE     0x0002U
#define SPANSION_S29GL064N_ID1                  0x227EU
#define SPANSION_S29GL064N_ID2                  0x2213U
#define SPANSION_S29GL064N_ID3                  0x2201U

/* Macronix MX28F4000 */
#define MX_FLASH_SECTORS         0x0020U
#define MX_SECTOR_SIZE           0x8000U
#define MX_PROG_SECTOR_SIZE      0x0002U

/* ATMEL AT29C040 */
#define AT_FLASH_SECTORS         0x0400U
#define AT_SECTOR_SIZE           0x0400U
#define AT_PROG_SECTOR_SIZE      AT_SECTOR_SIZE

/* ATMEL AT29C040A */
#define AT_A_FLASH_SECTORS       0x0800U
#define AT_A_SECTOR_SIZE         0x0200U
#define AT_A_PROG_SECTOR_SIZE    AT_A_SECTOR_SIZE

/* Silicon Storage Technology 28SF040 */
#define SST_FLASH_SECTORS        0x0800U
#define SST_SECTOR_SIZE          0x0200U
#define SST_PROG_SECTOR_SIZE     0x0002U

/* SHARP LH28F800SG */
#define SHARP_FLASH_SECTORS        0x0010U
#define SHARP_SECTOR_SIZE          0x10000U
#define SHARP_PROG_SECTOR_SIZE     0x0002U

/* This is used to identifly flash with non-uniform sector sizes */
#define VARIABLE_SECTOR_SIZE      0x0000U

/* AMD Am29F800B */
#define AMD_FLASH_SECTORS         0x0013U
#define AMD_SECTOR_SIZE           VARIABLE_SECTOR_SIZE
#define AMD_PROG_SECTOR_SIZE      0x0002U

/* AMD Am29LV160T/B, FUJITSU MBM29LV160, STM M29W160 */
#define	AM29LV160_SECTORS				0x0023U
#define AM29LV160T_ID					0x22C4U
#define AM29LV160B_ID					0x2249U

/* AMD Am29LV640, Am29LV641	*/
#define	AM29LV641_SECTORS				0x0080U
#define AM29LV641_ID					0x22D7U
#define AM29LV641_SECTOR_SIZE			0x10000

/* FUJITSU M29W641DL and M29W641DH (AMD AM29LV641 Compatible) */
#define M29W641D_ID						0x22C7U

/* AMD MirrorBit Tech Am29LV641ML/H (AMD AM29LV641 Compatible) */
#define AM29LV641M_ID1 					0x227EU
#define AM29LV641M_ID2 					0x2213U

/* AMD MirrorBit Tech Am29LV128M */
#define	AM29LV128M_SECTORS				0x0100U
#define AM29LV128M_ID1 					0x227EU
#define AM29LV128M_ID2 					0x2212U
#define AM29LV128M_SECTOR_SIZE			0x10000

/* FUJITSU 29LV800BA */
#define FUJITSU_FLASH_SECTORS         0x0013U
#define FUJITSU_SECTOR_SIZE           VARIABLE_SECTOR_SIZE
#define FUJITSU_PROG_SECTOR_SIZE      0x0002U

/* ATMEL AT49BV8011 */
#define AT_49BV8011_FLASH_SECTORS         0x0016U
#define AT_49BV8011_SECTOR_SIZE           VARIABLE_SECTOR_SIZE
#define AT_49BV8011_PROG_SECTOR_SIZE      0x0002U

/* ATMEL AT49BV1614A */
#define AT_49BV1614A_FLASH_SECTORS         0x0027U
#define AT_49BV1614A_SECTOR_SIZE           VARIABLE_SECTOR_SIZE
#define AT_49BV1614A_PROG_SECTOR_SIZE      0x0002U

/* Silicon Storage Technology 39VF800 */
#define SST_39VF800_FLASH_SECTORS        0x010U /* We are using block instead of sector */
#define SST_39VF800_FLASH_SECTOR_SIZE 	VARIABLE_SECTOR_SIZE
#define SST_39VF800_PROG_SECTOR_SIZE    0x0002U

/* ST Micro M29W800AB*/
#define STM_M29W800AB_FLASH_SECTORS        0x013U       /* We are using block instead of sector */
#define STM_M29W800AB_FLASH_SECTOR_SIZE 	VARIABLE_SECTOR_SIZE
#define STM_M29W800AB_PROG_SECTOR_SIZE    0x0002U

/* ST Micro M29W160DB*/
#define STM_M29W160DB_FLASH_SECTORS        0x023U       /* We are using block instead of sector */
#define STM_M29W160DB_FLASH_SECTOR_SIZE 	VARIABLE_SECTOR_SIZE
#define STM_M29W160DB_PROG_SECTOR_SIZE    0x0002U

/* ST Micro M29W320DB and M29W32EB*/
#define STM_M29W320DB_FLASH_SECTORS        0x043U       /* We are using block instead of sector */
#define STM_M29W320EB_FLASH_SECTORS        0x047U       /* We are using block instead of sector */
#define STM_M29W320xB_FLASH_SECTOR_SIZE 	VARIABLE_SECTOR_SIZE
#define STM_M29W320xB_PROG_SECTOR_SIZE    0x0002U

/* ST Micro M29DW323DB */
#define STM_M29DW323DB_FLASH_SECTORS        0x047U      /* Use both banks */
#define STM_M29DW323DB_FLASH_SECTOR_SIZE    VARIABLE_SECTOR_SIZE
#define STM_M29DW323DB_PROG_SECTOR_SIZE     0x0002U

/* ST Micro M29DW641F */
#define STM_M29DW641F_FLASH_SECTORS        0x08EU      /* Use both banks */
#define STM_M29DW641F_FLASH_SECTOR_SIZE    VARIABLE_SECTOR_SIZE
#define STM_M29DW641F_PROG_SECTOR_SIZE     0x0002U

    
/* AMD29DL323DB */
#define AMD_29DL323D_FLASH_SECTORS          0x047U      /* Use both banks */
#define AMD_29DL323D_FLASH_SECTOR_SIZE      VARIABLE_SECTOR_SIZE
#define AMD_29DL323D_PROG_SECTOR_SIZE       0x0002U

/* Spansion S29AL032D */
#define SPANSION_S29AL032D_FLASH_SECTORS        0x47U
#define SPANSION_S29AL032D_FLASH_SECTOR_SIZE    VARIABLE_SECTOR_SIZE
#define SPANSION_S29AL032D_PROG_SECTOR_SIZE     0x0002U
#define SPANSION_S29AL032D_ID1                  0x22F9U

/* Eon EN29LV320B */
#define EON_EN29LV320B_FLASH_SECTORS            0x47U
#define EON_EN29LV320B_FLASH_SECTOR_SIZE        VARIABLE_SECTOR_SIZE
#define EON_EN29LV320B_PROG_SECTOR_SIZE         0x0002U
#define EON_EN29LV320B_ID1                      0x22F9U

/* Spansion S29GL032N */
#define SPANSION_S29GL032N_FLASH_SECTORS        0x47U
#define SPANSION_S29GL032N_FLASH_SECTOR_SIZE    VARIABLE_SECTOR_SIZE
#define SPANSION_S29GL032N_PROG_SECTOR_SIZE     0x0002U
#define SPANSION_S29GL032N_ID1                  0x227EU

/*ad*
 * These are the return values for the old flash driver API.
 *
 * @external
 * @include     <Flash.h>
 * @category Flash_Memory
 * @since 6.0
 * @name FLASH_ERROR    "Legacy Flash Driver Error Codes"
 *
 * @param  UNKNOWN_FLASH            -1
 * @param  ERASE_FAILED             -2
 * @param  WRITE_FAILED             -3
 * @param  VERIFY_FAILED            -4
 * @param  SECTORNUMBER_INVALID     -5
 * @param  MEMORY_OUTOFBOUND        -6
*ad*/
#define UNKNOWN_FLASH            -1
#define ERASE_FAILED             -2
#define WRITE_FAILED             -3
#define VERIFY_FAILED            -4
#define SECTORNUMBER_INVALID     -5
#define MEMORY_OUTOFBOUND        -6

/*ad*
 * These are the return values for the new flash driver API.
 *
 * @external
 * @include     <Flash.h>
 * @category Flash_Memory
 * @since 6.0
 * @name NAFLASH    "Flash Driver Error Codes"
 *
 * @param  NAFLASH_SUCCESS                  0
 * @param  NAFLASH_UNKNOWN_FLASH           UNKNOWN_FLASH
 * @param  NAFLASH_ERASE_FAILED            ERASE_FAILED
 * @param  NAFLASH_WRITE_FAILED            WRITE_FAILED
 * @param  NAFLASH_VERIFY_FAILED           VERIFY_FAILED
 * @param  NAFLASH_SECTORNUMBER_INVALID    SECTORNUMBER_INVALID
 * @param  NAFLASH_MEMORY_OUTOFBOUND       MEMORY_OUTOFBOUND
 * @param  NAFLASH_SECTOROFFSET_INVALID    -7
 * @param  NAFLASH_WRITEOPTION_INVALID     -8
 * @param  NAFLASH_FLASHBANKS_INVALID      -9
 * @param  NAFLASH_DUPLICATE_CALL          -10
 * @param  NAFLASH_SEMAPHORE_CREATE_FAILED -11
 *
 * @see @link FLASH_ERROR
*ad*/
#define NAFLASH_SUCCESS                  0
#define NAFLASH_UNKNOWN_FLASH           UNKNOWN_FLASH
#define NAFLASH_ERASE_FAILED            ERASE_FAILED
#define NAFLASH_WRITE_FAILED            WRITE_FAILED
#define NAFLASH_VERIFY_FAILED           VERIFY_FAILED
#define NAFLASH_SECTORNUMBER_INVALID    SECTORNUMBER_INVALID
#define NAFLASH_MEMORY_OUTOFBOUND       MEMORY_OUTOFBOUND
#define NAFLASH_SECTOROFFSET_INVALID    -7
#define NAFLASH_WRITEOPTION_INVALID     -8
#define NAFLASH_FLASHBANKS_INVALID      -9
#define NAFLASH_DUPLICATE_CALL          -10
#define NAFLASH_SEMAPHORE_CREATE_FAILED -11

/* Maximum number of flash sectors supported.
   The default value of 142 allows supporting
   two banks of 4MB flash parts that have
   variable sector sizes (i.e. AMD29DL323).
   Increase value of MAX_SECTORS to support
   flash parts with a larger sector count. */
#define MAX_SECTORS                   256       /* HB: Increased to 256 for AM29LV128M */

/* Maximum sector size supported.
   Increase value of MAX_SECTOR_SIZE to
   support flash parts with a larger
   sector size. */
#define MAX_SECTOR_SIZE              128 * 1024

/* Maximum flash memory banks supported.
   Increase value of MAX_FLASH_BANKS to
   support a larger number of flash
   banks */
#define MAX_FLASH_BANKS              2

/* ROM type values */

#define SHARP_FLASH                  0x00
#define ATMEL_29C040                 0x01
#define ATMEL_29C040A                0x02
#define MX_32K_FLASH                 0x03
#define SST_FLASH                    0x04
#define AMD_B_5V_FLASH_ID            0x05
#define AMD_B_3V_FLASH_ID            0x06
#define AMD_T_5V_FLASH_ID            0x07
#define AMD_T_3V_FLASH_ID            0x08
#define FUJITSU_B_3V_FLASH_ID        0x09
#define ATMEL_49BV8011               0x0A
#define ATMEL_49BV8011T              0x0B
#define SST_39VF800                  0x0C
#define STM_29W800AB                 0x0D
#define AMD_29DL323D                 0x0E
#define ATMEL_49BV1614A			 	 0x0F

/*ad*
 * These are the NAFlashWrite() options.
 *
 * @external
 * @include     <Flash.h>
 * @category Flash_Memory
 * @since 6.0
 * @name FLASH_WRITE_OPTION    "NAFlashWrite() Options"
 *
 * @param  ERASE_AS_NEEDED        0
 * @param  ALWAYS_ERASE           1
 * @param  DO_NOT_ERASE           2
*ad*/
#define ERASE_AS_NEEDED        0
#define ALWAYS_ERASE           1
#define DO_NOT_ERASE           2

    typedef struct
    {
        WORD8 ccode;            /* command code */
        WORD32 ccode_addr;      /* address of command code */
    } flash_cmd_t;

    typedef struct
    {
        WORD16 mcode;           /* manufacture code */
        WORD16 mcode_addr;      /* address of manufacture code */
        WORD16 dcode;           /* device code */
        WORD16 dcode_addr;      /* address of device code */
        WORD16 dcode2;          /* device code 2 */
        WORD16 dcode2_addr;     /* address of device code 2 */
        WORD16 dcode3;          /* device code 2 */
        WORD16 dcode3_addr;     /* address of device code 2 */
        WORD16 total_sector_number;     /* total number of sector */
        WORD32 sector_size;     /* size of a sectors (in bytes) */
        WORD16 prog_size;       /* program load size (in bytes) */
        WORD16 access_time;     /* access time in nanoseconds */
        flash_cmd_t *id_enter_cmd;      /* enter identify flash command */
        WORD16 id_enter_len;    /* number of cycles for above command */
        flash_cmd_t *id_exit_cmd;       /* exit identify flash command */
        WORD16 id_exit_len;     /* number of cycles for above command */
        flash_cmd_t *erase_cmd; /* erase flash command */
        WORD16 erase_len;       /* number of cycles for above command */
        flash_cmd_t *write_cmd; /* write flash command */
        WORD16 write_len;       /* number of cycles for above command */
        flash_cmd_t *sector_erase_cmd;  /* for AMD only */
        WORD32 *sector_size_array;      /* For non-uniform sector sizes */
    } flash_id_table_t;

extern const flash_id_table_t *naflash_id_table;

extern const WORD8 naflash_num_id;

#ifdef __cplusplus
}
#endif
#endif
