/*****************************************************************************
* Copyright (c) 2005 Digi International Inc., All Rights Reserved
* 
* This software contains proprietary and confidential information of Digi
* International Inc.  By accepting transfer of this copy, Recipient agrees
* to retain this software in confidence, to prevent disclosure to others,
* and to make no use of this software other than that for which it was
* delivered.  This is an unpublished copyrighted work of Digi International
* Inc.  Except as permitted by federal law, 17 USC 117, copying is strictly
* prohibited.
* 
* Restricted Rights Legend
*
* Use, duplication, or disclosure by the Government is subject to
* restrictions set forth in sub-paragraph (c)(1)(ii) of The Rights in
* Technical Data and Computer Software clause at DFARS 252.227-7031 or
* subparagraphs (c)(1) and (2) of the Commercial Computer Software -
* Restricted Rights at 48 CFR 52.227-19, as applicable.
*
* Digi International Inc. 11001 Bren Road East, Minnetonka, MN 55343
*
*****************************************************************************/
#include "naflash.h"
#include "flash.h"
#include "flashparts.h"

/*
 * This file is part of the NET+OS flash driver.  It contains the command
 * codes used to control flash parts along with the flash part's manufacturer
 * and device IDs, and sector sizes.
 *
 * This code is located in the BSP directory instead of src/flash because 
 * the data structures in this file are conditionally compiled to only include
 * information on the parts that are actually supported by the hardware.  This
 * will be BSP platform specific.  The supported parts should be selected by
 * editing the file flashparts.h in the platform directory.
 */
#if NAFLASH_WANT_TO_SUPPORT_MX28F4000
/*
 *
 *  Commands for Macronix Flash
 *
 */

static const flash_cmd_t MX_flash_id_cmd[] = {
    {0x90, 0x0000}
};

static const flash_cmd_t MX_flash_erase_cmd[] = {
    {0x20, 0x0000}, {0xD0, 0x0000}
};

static const flash_cmd_t MX_flash_write_cmd[] = {
    {0x40, 0x0000}
};
#endif

#define NEED_ATMEL_COMMANDS (NAFLASH_WANT_TO_SUPPORT_AT29C040 || NAFLASH_WANT_TO_SUPPORT_AT29C040A || NAFLASH_WANT_TO_SUPPORT_AT49BV8011 || NAFLASH_WANT_TO_SUPPORT_AT49BV8011T || NAFLASH_WANT_TO_SUPPORT_AT49BV1614A)

#if NEED_ATMEL_COMMANDS
/*
 *
 *  Commands for Atmel Flash
 *
 */

static const flash_cmd_t AT_flash_id_enter_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x90, 0x5555}
};

static const flash_cmd_t AT_flash_id_exit_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xF0, 0x5555}
};

static const flash_cmd_t AT_flash_write_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xA0, 0x5555}
};

static const flash_cmd_t AT_flash_erase_cmd[] = {
    {0xAA, 0x5555},
    {0x55, 0x2AAA},
    {0x80, 0x5555},
    {0xAA, 0x5555},
    {0x55, 0x2AAA}
};
#endif

#if NAFLASH_WANT_TO_SUPPORT_AT49BV8011
/*
 * Array of commands user to erase sectors, starting with the first.
 */
static const flash_cmd_t AT_flash_AT49BV8011_sector_erase_cmd[] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x02000},
    {0x30, 0x06000},
    {0x30, 0x07000},
    {0x30, 0x08000},
    {0x30, 0x09000},
    {0x30, 0x0a000},
    {0x30, 0x0e000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000}
};

/* Array of all the sector sizes */
static const WORD32 AT_AT49BV8011_sector_size_array[] = {
    0x4000, 0x8000,
    0x2000, 0x2000, 0x2000, 0x2000, 0x8000, 0x4000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000
};
#endif

#if NAFLASH_WANT_TO_SUPPORT_AT49BV8011T
/*
 * Array of commands user to erase sectors, starting with the first.
*/
static const flash_cmd_t AT_flash_AT49BV8011T_sector_erase_cmd[] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x08000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x72000},
    {0x30, 0x76000},
    {0x30, 0x77000},
    {0x30, 0x78000},
    {0x30, 0x79000},
    {0x30, 0x7a000},
    {0x30, 0x7e000}
};

/* Array of all the sector sizes */

static const WORD32 AT_AT49BV8011T_sector_size_array[] = {
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x4000, 0x8000,
    0x2000, 0x2000, 0x2000, 0x2000, 0x8000, 0x4000
};
#endif

#if NAFLASH_WANT_TO_SUPPORT_AT49BV1614A
/* 
 *  AT49BV1614A 2 Meg Bytes Flash Memory 
 *  Sectors Size:   8 Sectors 8K Bytes  
 *                  31 Sectors 64K Bytes 
 */

/*
 * Flash Array of commands user to erase sectors, starting with the first.
 */
static const flash_cmd_t AT_flash_AT49BV1614A_sector_erase_cmd[] = {
    {0x30, 0x00000}, {0x30, 0x01000}, {0x30, 0x02000}, {0x30, 0x03000}, {0x30, 0x04000}, {0x30, 0x05000},
    {0x30, 0x06000}, {0x30, 0x07000}, {0x30, 0x08000}, {0x30, 0x10000}, {0x30, 0x18000}, {0x30, 0x20000},
    {0x30, 0x28000}, {0x30, 0x30000}, {0x30, 0x38000}, {0x30, 0x40000}, {0x30, 0x48000}, {0x30, 0x50000},
    {0x30, 0x58000}, {0x30, 0x60000}, {0x30, 0x68000}, {0x30, 0x70000}, {0x30, 0x78000}, {0x30, 0x80000},
    {0x30, 0x88000}, {0x30, 0x90000}, {0x30, 0x98000}, {0x30, 0xA0000}, {0x30, 0xA8000}, {0x30, 0xB0000},
    {0x30, 0xB8000}, {0x30, 0xC0000}, {0x30, 0xC8000}, {0x30, 0xD0000}, {0x30, 0xD8000}, {0x30, 0xE0000},
    {0x30, 0xE8000}, {0x30, 0xF0000}, {0x30, 0xF8000},
};

/* AT49BV1614A  Array of all the sector sizes */
static const WORD32 AT_AT49BV1614A_sector_size_array[] = {
    0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000,
    0x2000, 0x2000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000,
};
#endif

#if NAFLASH_WANT_TO_SUPPORT_SST28SF040
/*
 *
 *  Commands for SST Flash
 *
 */

static const flash_cmd_t SST_flash_id_enter_cmd[] = {
    {0x90, 0x0000}
};

static const flash_cmd_t SST_flash_id_exit_cmd[] = {
    {0xFF, 0x0000}
};

static const flash_cmd_t SST_flash_erase_cmd[] = {
    {0x20, 0x0000}, {0xD0, 0x0000}
};

static const flash_cmd_t SST_flash_write_cmd[] = {
    {0x10, 0x0000}
};
#endif

#if NAFLASH_WANT_TO_SUPPORT_SST39VF800
/*
 *
 *  Commands for SST39VF800
 *
 *
 */

static const flash_cmd_t SST_39VF800_flash_id_enter_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x90, 0x5555}
};

static const flash_cmd_t SST_39VF800_flash_id_exit_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xF0, 0x5555}
};

static const flash_cmd_t SST_39VF800_flash_erase_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x80, 0x5555},
    {0xAA, 0x5555}, {0x55, 0x2AAA}
};

static const flash_cmd_t SST_39VF800_flash_write_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xA0, 0x5555}
};

/*
 * Array of commands user to erase sectors, starting with the first.
 */
static const flash_cmd_t SST_39VF800_flash_block_erase_cmd[16] = {
    {0x50, 0x00000},            /* Command to erase the first sector */
    {0x50, 0x08000},
    {0x50, 0x10000},
    {0x50, 0x18000},
    {0x50, 0x20000},
    {0x50, 0x28000},
    {0x50, 0x30000},
    {0x50, 0x38000},
    {0x50, 0x40000},
    {0x50, 0x48000},
    {0x50, 0x50000},
    {0x50, 0x58000},
    {0x50, 0x60000},
    {0x50, 0x68000},
    {0x50, 0x70000},
    {0x50, 0x78000}
};
#endif

#if NAFLASH_WANT_TO_SUPPORT_SST39VF800
/* Array of all the sector sizes */

/* Although the SST39VF800 has uniform block size, it has different erase command for
 * each block.  So we are treating it as flash with variable sector size.  In addition
 * we also use 64k byte block instead of the 2k sector size when working with this flash.
 */

static const WORD32 SST_39VF800_flash_block_size_array[] = { 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000
};
#endif

#if NAFLASH_WANT_TO_SUPPORT_LH28F800SG
/*
 *
 *  Commands for SHARP Flash
 *
 */

static const flash_cmd_t SHARP_flash_id_enter_cmd[] = {
    {0x90, 0x0000}
};

static const flash_cmd_t SHARP_flash_id_exit_cmd[] = {
    {0xFF, 0x0000}
};

static const flash_cmd_t SHARP_flash_erase_cmd[] = {
    {0x20, 0x0000}, {0xD0, 0x0000}
};

static const flash_cmd_t SHARP_flash_write_cmd[] = {
    {0x10, 0x0000}
};
#endif


#define NEED_AMD_BOTTOM  (NAFLASH_WANT_TO_SUPPORT_AM29F800BB || NAFLASH_WANT_TO_SUPPORT_AM29LV800BB || NAFLASH_WANT_TO_SUPPORT_MBM29LV800BE)
#define NEED_AMD_TOP     (NAFLASH_WANT_TO_SUPPORT_AM29F800BT || NAFLASH_WANT_TO_SUPPORT_AM29LV800BT || NAFLASH_WANT_TO_SUPPORT_AM29LV641 || NAFLASH_WANT_TO_SUPPORT_AM29LV160T)

#if (NEED_AMD_TOP || NEED_AMD_BOTTOM)
/*
 * Command sequences for the AMD Am29F800B
 */
static const flash_cmd_t AMD_flash_id_enter_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x90, 0x5555}
};

static const flash_cmd_t AMD_flash_id_exit_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xF0, 0x5555}
};

static const flash_cmd_t AMD_flash_write_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xA0, 0x5555}
};

static const flash_cmd_t AMD_flash_erase_cmd[] = {
    {0xAA, 0x5555},
    {0x55, 0x2AAA},
    {0x80, 0x5555},
    {0xAA, 0x5555},
    {0x55, 0x2AAA}
};


#if NEED_AMD_TOP

#if (NAFLASH_WANT_TO_SUPPORT_AM29F800BT || NAFLASH_WANT_TO_SUPPORT_MBM29LV800BE)
/*
 * Array of commands user to erase sectors, starting with the first.
 */
static const flash_cmd_t AMD_flash_top_sector_erase_cmd[20] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x08000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000},
    {0x30, 0x7c000},
    {0x30, 0x7d000},
    {0x30, 0x7e000}
};

static const WORD32 amd_top_sector_size_array[] = { 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x8000, 0x2000, 0x2000, 0x4000
};
#endif

#endif

#if NEED_AMD_BOTTOM
/*
 * Array of commands user to erase sectors, starting with the first.
 */
static const flash_cmd_t AMD_flash_bottom_sector_erase_cmd[20] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x02000},
    {0x30, 0x03000},
    {0x30, 0x04000},
    {0x30, 0x08000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000}
};

/* Array of all the sector sizes */
static const WORD32 amd_bottom_sector_size_array[] = { 0x4000, 0x2000, 0x2000, 0x8000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000
};
#endif
#endif


#if (NAFLASH_WANT_TO_SUPPORT_M29W160T || NAFLASH_WANT_TO_SUPPORT_MBM29LV160T || NAFLASH_WANT_TO_SUPPORT_AM29LV160T)
static const flash_cmd_t AMD_29lv160_top_sector_erase_cmd[36] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x08000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000},
    {0x30, 0x80000},
    {0x30, 0x88000},
    {0x30, 0x90000},
    {0x30, 0x98000},
    {0x30, 0xa0000},
    {0x30, 0xa8000},
    {0x30, 0xb0000},
    {0x30, 0xb8000},
    {0x30, 0xc0000},
    {0x30, 0xc8000},
    {0x30, 0xd0000},
    {0x30, 0xd8000},
    {0x30, 0xe0000},
    {0x30, 0xe8000},
    {0x30, 0xf0000},
    {0x30, 0xf8000},
    {0x30, 0xfc000},
    {0x30, 0xfd000},
    {0x30, 0xfe000}
};

static const WORD32 amd_29lv160_top_sector_size_array[] =
    { 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x8000, 0x2000, 0x2000, 0x4000
};
#endif



#if (NAFLASH_WANT_TO_SUPPORT_AM29LV160B || NAFLASH_WANT_TO_SUPPORT_MBM29LV160B || NAFLASH_WANT_TO_SUPPORT_M29W160B)

static const WORD32 amd_29lv160_bottom_sector_size_array[] = { 0x4000, 0x2000, 0x2000, 0x8000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000
};


static const flash_cmd_t AMD_29lv160_bottom_sector_erase_cmd[36] = {
    {0x30, 0x00000},
    {0x30, 0x02000},
    {0x30, 0x03000},
    {0x30, 0x04000}, {0x30, 0x08000}, {0x30, 0x10000}, {0x30, 0x18000},
    {0x30, 0x20000}, {0x30, 0x28000}, {0x30, 0x30000}, {0x30, 0x38000},
    {0x30, 0x40000}, {0x30, 0x48000}, {0x30, 0x50000}, {0x30, 0x58000},
    {0x30, 0x60000}, {0x30, 0x68000}, {0x30, 0x70000}, {0x30, 0x78000},
    {0x30, 0x80000}, {0x30, 0x88000}, {0x30, 0x90000}, {0x30, 0x98000},
    {0x30, 0xa0000}, {0x30, 0xa8000}, {0x30, 0xb0000}, {0x30, 0xb8000},
    {0x30, 0xc0000}, {0x30, 0xc8000}, {0x30, 0xd0000}, {0x30, 0xd8000},
    {0x30, 0xe0000}, {0x30, 0xe8000}, {0x30, 0xf0000}, {0x30, 0xf8000}
};
#endif


#define NEED_AM29LV641MH_COMPATIBLE (NAFLASH_WANT_TO_SUPPORT_AM29LV641MH || NAFLASH_WANT_TO_SUPPORT_M29W641DL || NAFLASH_WANT_TO_SUPPORT_M29W641DH || NAFLASH_WANT_TO_SUPPORT_MBM29LV650U || NAFLASH_WANT_TO_SUPPORT_MX29LV640BUTI90 || NAFLASH_WANT_TO_SUPPORT_AM29LV641)

#if NEED_AM29LV641MH_COMPATIBLE
static const flash_cmd_t AMD_29lv641_sector_erase_cmd[0x80] = {
    {0x30, 0x000000}, {0x30, 0x008000}, {0x30, 0x010000}, {0x30, 0x018000},
    {0x30, 0x020000}, {0x30, 0x028000}, {0x30, 0x030000}, {0x30, 0x038000},
    {0x30, 0x040000}, {0x30, 0x048000}, {0x30, 0x050000}, {0x30, 0x058000},
    {0x30, 0x060000}, {0x30, 0x068000}, {0x30, 0x070000}, {0x30, 0x078000},
    {0x30, 0x080000}, {0x30, 0x088000}, {0x30, 0x090000}, {0x30, 0x098000},
    {0x30, 0x0a0000}, {0x30, 0x0a8000}, {0x30, 0x0b0000}, {0x30, 0x0b8000},
    {0x30, 0x0c0000}, {0x30, 0x0c8000}, {0x30, 0x0d0000}, {0x30, 0x0d8000},
    {0x30, 0x0e0000}, {0x30, 0x0e8000}, {0x30, 0x0f0000}, {0x30, 0x0f8000},

    {0x30, 0x100000}, {0x30, 0x108000}, {0x30, 0x110000}, {0x30, 0x118000},
    {0x30, 0x120000}, {0x30, 0x128000}, {0x30, 0x130000}, {0x30, 0x138000},
    {0x30, 0x140000}, {0x30, 0x148000}, {0x30, 0x150000}, {0x30, 0x158000},
    {0x30, 0x160000}, {0x30, 0x168000}, {0x30, 0x170000}, {0x30, 0x178000},
    {0x30, 0x180000}, {0x30, 0x188000}, {0x30, 0x190000}, {0x30, 0x198000},
    {0x30, 0x1a0000}, {0x30, 0x1a8000}, {0x30, 0x1b0000}, {0x30, 0x1b8000},
    {0x30, 0x1c0000}, {0x30, 0x1c8000}, {0x30, 0x1d0000}, {0x30, 0x1d8000},
    {0x30, 0x1e0000}, {0x30, 0x1e8000}, {0x30, 0x1f0000}, {0x30, 0x1f8000},

    {0x30, 0x200000}, {0x30, 0x208000}, {0x30, 0x210000}, {0x30, 0x218000},
    {0x30, 0x220000}, {0x30, 0x228000}, {0x30, 0x230000}, {0x30, 0x238000},
    {0x30, 0x240000}, {0x30, 0x248000}, {0x30, 0x250000}, {0x30, 0x258000},
    {0x30, 0x260000}, {0x30, 0x268000}, {0x30, 0x270000}, {0x30, 0x278000},
    {0x30, 0x280000}, {0x30, 0x288000}, {0x30, 0x290000}, {0x30, 0x298000},
    {0x30, 0x2a0000}, {0x30, 0x2a8000}, {0x30, 0x2b0000}, {0x30, 0x2b8000},
    {0x30, 0x2c0000}, {0x30, 0x2c8000}, {0x30, 0x2d0000}, {0x30, 0x2d8000},
    {0x30, 0x2e0000}, {0x30, 0x2e8000}, {0x30, 0x2f0000}, {0x30, 0x2f8000},

    {0x30, 0x300000}, {0x30, 0x308000}, {0x30, 0x310000}, {0x30, 0x318000},
    {0x30, 0x320000}, {0x30, 0x328000}, {0x30, 0x330000}, {0x30, 0x338000},
    {0x30, 0x340000}, {0x30, 0x348000}, {0x30, 0x350000}, {0x30, 0x358000},
    {0x30, 0x360000}, {0x30, 0x368000}, {0x30, 0x370000}, {0x30, 0x378000},
    {0x30, 0x380000}, {0x30, 0x388000}, {0x30, 0x390000}, {0x30, 0x398000},
    {0x30, 0x3a0000}, {0x30, 0x3a8000}, {0x30, 0x3b0000}, {0x30, 0x3b8000},
    {0x30, 0x3c0000}, {0x30, 0x3c8000}, {0x30, 0x3d0000}, {0x30, 0x3d8000},
    {0x30, 0x3e0000}, {0x30, 0x3e8000}, {0x30, 0x3f0000}, {0x30, 0x3f8000}
};
#endif

#if NAFLASH_WANT_TO_SUPPORT_AM29LV128M
static const flash_cmd_t AMD_29lv128M_sector_erase_cmd[0x100] = {
    {0x30, 0x000000}, {0x30, 0x008000}, {0x30, 0x010000}, {0x30, 0x018000},
    {0x30, 0x020000}, {0x30, 0x028000}, {0x30, 0x030000}, {0x30, 0x038000},
    {0x30, 0x040000}, {0x30, 0x048000}, {0x30, 0x050000}, {0x30, 0x058000},
    {0x30, 0x060000}, {0x30, 0x068000}, {0x30, 0x070000}, {0x30, 0x078000},
    {0x30, 0x080000}, {0x30, 0x088000}, {0x30, 0x090000}, {0x30, 0x098000},
    {0x30, 0x0a0000}, {0x30, 0x0a8000}, {0x30, 0x0b0000}, {0x30, 0x0b8000},
    {0x30, 0x0c0000}, {0x30, 0x0c8000}, {0x30, 0x0d0000}, {0x30, 0x0d8000},
    {0x30, 0x0e0000}, {0x30, 0x0e8000}, {0x30, 0x0f0000}, {0x30, 0x0f8000},

    {0x30, 0x100000}, {0x30, 0x108000}, {0x30, 0x110000}, {0x30, 0x118000},
    {0x30, 0x120000}, {0x30, 0x128000}, {0x30, 0x130000}, {0x30, 0x138000},
    {0x30, 0x140000}, {0x30, 0x148000}, {0x30, 0x150000}, {0x30, 0x158000},
    {0x30, 0x160000}, {0x30, 0x168000}, {0x30, 0x170000}, {0x30, 0x178000},
    {0x30, 0x180000}, {0x30, 0x188000}, {0x30, 0x190000}, {0x30, 0x198000},
    {0x30, 0x1a0000}, {0x30, 0x1a8000}, {0x30, 0x1b0000}, {0x30, 0x1b8000},
    {0x30, 0x1c0000}, {0x30, 0x1c8000}, {0x30, 0x1d0000}, {0x30, 0x1d8000},
    {0x30, 0x1e0000}, {0x30, 0x1e8000}, {0x30, 0x1f0000}, {0x30, 0x1f8000},

    {0x30, 0x200000}, {0x30, 0x208000}, {0x30, 0x210000}, {0x30, 0x218000},
    {0x30, 0x220000}, {0x30, 0x228000}, {0x30, 0x230000}, {0x30, 0x238000},
    {0x30, 0x240000}, {0x30, 0x248000}, {0x30, 0x250000}, {0x30, 0x258000},
    {0x30, 0x260000}, {0x30, 0x268000}, {0x30, 0x270000}, {0x30, 0x278000},
    {0x30, 0x280000}, {0x30, 0x288000}, {0x30, 0x290000}, {0x30, 0x298000},
    {0x30, 0x2a0000}, {0x30, 0x2a8000}, {0x30, 0x2b0000}, {0x30, 0x2b8000},
    {0x30, 0x2c0000}, {0x30, 0x2c8000}, {0x30, 0x2d0000}, {0x30, 0x2d8000},
    {0x30, 0x2e0000}, {0x30, 0x2e8000}, {0x30, 0x2f0000}, {0x30, 0x2f8000},

    {0x30, 0x300000}, {0x30, 0x308000}, {0x30, 0x310000}, {0x30, 0x318000},
    {0x30, 0x320000}, {0x30, 0x328000}, {0x30, 0x330000}, {0x30, 0x338000},
    {0x30, 0x340000}, {0x30, 0x348000}, {0x30, 0x350000}, {0x30, 0x358000},
    {0x30, 0x360000}, {0x30, 0x368000}, {0x30, 0x370000}, {0x30, 0x378000},
    {0x30, 0x380000}, {0x30, 0x388000}, {0x30, 0x390000}, {0x30, 0x398000},
    {0x30, 0x3a0000}, {0x30, 0x3a8000}, {0x30, 0x3b0000}, {0x30, 0x3b8000},
    {0x30, 0x3c0000}, {0x30, 0x3c8000}, {0x30, 0x3d0000}, {0x30, 0x3d8000},
    {0x30, 0x3e0000}, {0x30, 0x3e8000}, {0x30, 0x3f0000}, {0x30, 0x3f8000},

    {0x30, 0x400000}, {0x30, 0x408000}, {0x30, 0x410000}, {0x30, 0x418000},
    {0x30, 0x420000}, {0x30, 0x428000}, {0x30, 0x430000}, {0x30, 0x438000},
    {0x30, 0x440000}, {0x30, 0x448000}, {0x30, 0x450000}, {0x30, 0x458000},
    {0x30, 0x460000}, {0x30, 0x468000}, {0x30, 0x470000}, {0x30, 0x478000},
    {0x30, 0x480000}, {0x30, 0x488000}, {0x30, 0x490000}, {0x30, 0x498000},
    {0x30, 0x4a0000}, {0x30, 0x4a8000}, {0x30, 0x4b0000}, {0x30, 0x4b8000},
    {0x30, 0x4c0000}, {0x30, 0x4c8000}, {0x30, 0x4d0000}, {0x30, 0x4d8000},
    {0x30, 0x4e0000}, {0x30, 0x4e8000}, {0x30, 0x4f0000}, {0x30, 0x4f8000},

    {0x30, 0x500000}, {0x30, 0x508000}, {0x30, 0x510000}, {0x30, 0x518000},
    {0x30, 0x520000}, {0x30, 0x528000}, {0x30, 0x530000}, {0x30, 0x538000},
    {0x30, 0x540000}, {0x30, 0x548000}, {0x30, 0x550000}, {0x30, 0x558000},
    {0x30, 0x560000}, {0x30, 0x568000}, {0x30, 0x570000}, {0x30, 0x578000},
    {0x30, 0x580000}, {0x30, 0x588000}, {0x30, 0x590000}, {0x30, 0x598000},
    {0x30, 0x5a0000}, {0x30, 0x5a8000}, {0x30, 0x5b0000}, {0x30, 0x5b8000},
    {0x30, 0x5c0000}, {0x30, 0x5c8000}, {0x30, 0x5d0000}, {0x30, 0x5d8000},
    {0x30, 0x5e0000}, {0x30, 0x5e8000}, {0x30, 0x5f0000}, {0x30, 0x5f8000},

    {0x30, 0x600000}, {0x30, 0x608000}, {0x30, 0x610000}, {0x30, 0x618000},
    {0x30, 0x620000}, {0x30, 0x628000}, {0x30, 0x630000}, {0x30, 0x638000},
    {0x30, 0x640000}, {0x30, 0x648000}, {0x30, 0x650000}, {0x30, 0x658000},
    {0x30, 0x660000}, {0x30, 0x668000}, {0x30, 0x670000}, {0x30, 0x678000},
    {0x30, 0x680000}, {0x30, 0x688000}, {0x30, 0x690000}, {0x30, 0x698000},
    {0x30, 0x6a0000}, {0x30, 0x6a8000}, {0x30, 0x6b0000}, {0x30, 0x6b8000},
    {0x30, 0x6c0000}, {0x30, 0x6c8000}, {0x30, 0x6d0000}, {0x30, 0x6d8000},
    {0x30, 0x6e0000}, {0x30, 0x6e8000}, {0x30, 0x6f0000}, {0x30, 0x6f8000},

    {0x30, 0x700000}, {0x30, 0x708000}, {0x30, 0x710000}, {0x30, 0x718000},
    {0x30, 0x720000}, {0x30, 0x728000}, {0x30, 0x730000}, {0x30, 0x738000},
    {0x30, 0x740000}, {0x30, 0x748000}, {0x30, 0x750000}, {0x30, 0x758000},
    {0x30, 0x760000}, {0x30, 0x768000}, {0x30, 0x770000}, {0x30, 0x778000},
    {0x30, 0x780000}, {0x30, 0x788000}, {0x30, 0x790000}, {0x30, 0x798000},
    {0x30, 0x7a0000}, {0x30, 0x7a8000}, {0x30, 0x7b0000}, {0x30, 0x7b8000},
    {0x30, 0x7c0000}, {0x30, 0x7c8000}, {0x30, 0x7d0000}, {0x30, 0x7d8000},
    {0x30, 0x7e0000}, {0x30, 0x7e8000}, {0x30, 0x7f0000}, {0x30, 0x7f8000}
};
#endif


#if NAFLASH_WANT_TO_SUPPORT_AM29DL323DT
/*
 * Array of commands user to erase sectors, starting with the first.
 */
static const flash_cmd_t AMD_29DL323D_flash_top_sector_erase_cmd[71] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x08000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000},
    {0x30, 0x80000},
    {0x30, 0x88000},
    {0x30, 0x90000},
    {0x30, 0x98000},
    {0x30, 0xA0000},
    {0x30, 0xA8000},
    {0x30, 0xB0000},
    {0x30, 0xB8000},
    {0x30, 0xC0000},
    {0x30, 0xC8000},
    {0x30, 0xD0000},
    {0x30, 0xD8000},
    {0x30, 0xE0000},
    {0x30, 0xE8000},
    {0x30, 0xF0000},
    {0x30, 0xF8000},
    {0x30, 0x100000},
    {0x30, 0x108000},
    {0x30, 0x110000},
    {0x30, 0x118000},
    {0x30, 0x120000},
    {0x30, 0x128000},
    {0x30, 0x130000},
    {0x30, 0x138000},
    {0x30, 0x140000},
    {0x30, 0x148000},
    {0x30, 0x150000},
    {0x30, 0x158000},
    {0x30, 0x160000},
    {0x30, 0x168000},
    {0x30, 0x170000},
    {0x30, 0x178000},
    {0x30, 0x180000},
    {0x30, 0x188000},
    {0x30, 0x190000},
    {0x30, 0x198000},
    {0x30, 0x1A0000},
    {0x30, 0x1A8000},
    {0x30, 0x1B0000},
    {0x30, 0x1B8000},
    {0x30, 0x1C0000},
    {0x30, 0x1C8000},
    {0x30, 0x1D0000},
    {0x30, 0x1D8000},
    {0x30, 0x1E0000},
    {0x30, 0x1E8000},
    {0x30, 0x1F0000},
    {0x30, 0x1F8000},
    {0x30, 0x1F9000},
    {0x30, 0x1FA000},
    {0x30, 0x1FB000},
    {0x30, 0x1FC000},
    {0x30, 0x1FD000},
    {0x30, 0x1FE000},
    {0x30, 0x1FF000}
};

static const WORD32 amd_29DL323D_top_sector_size_array[] = {
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000,
    0x2000, 0x2000, 0x2000, 0x2000,
    0x2000, 0x2000, 0x2000, 0x2000
};
#endif


#if (NAFLASH_WANT_TO_SUPPORT_AM29DL323DTB || NAFLASH_WANT_TO_SUPPORT_AM29LV320DB)
/*
 * Array of commands user to erase sectors, starting with the first.
 */
static const flash_cmd_t AMD_29DL323D_flash_bottom_sector_erase_cmd[71] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x01000},
    {0x30, 0x02000},
    {0x30, 0x03000},
    {0x30, 0x04000},
    {0x30, 0x05000},
    {0x30, 0x06000},
    {0x30, 0x07000},
    {0x30, 0x08000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000},
    {0x30, 0x80000},
    {0x30, 0x88000},
    {0x30, 0x90000},
    {0x30, 0x98000},
    {0x30, 0xA0000},
    {0x30, 0xA8000},
    {0x30, 0xB0000},
    {0x30, 0xB8000},
    {0x30, 0xC0000},
    {0x30, 0xC8000},
    {0x30, 0xD0000},
    {0x30, 0xD8000},
    {0x30, 0xE0000},
    {0x30, 0xE8000},
    {0x30, 0xF0000},
    {0x30, 0xF8000},
    {0x30, 0x100000},
    {0x30, 0x108000},
    {0x30, 0x110000},
    {0x30, 0x118000},
    {0x30, 0x120000},
    {0x30, 0x128000},
    {0x30, 0x130000},
    {0x30, 0x138000},
    {0x30, 0x140000},
    {0x30, 0x148000},
    {0x30, 0x150000},
    {0x30, 0x158000},
    {0x30, 0x160000},
    {0x30, 0x168000},
    {0x30, 0x170000},
    {0x30, 0x178000},
    {0x30, 0x180000},
    {0x30, 0x188000},
    {0x30, 0x190000},
    {0x30, 0x198000},
    {0x30, 0x1A0000},
    {0x30, 0x1A8000},
    {0x30, 0x1B0000},
    {0x30, 0x1B8000},
    {0x30, 0x1C0000},
    {0x30, 0x1C8000},
    {0x30, 0x1D0000},
    {0x30, 0x1D8000},
    {0x30, 0x1E0000},
    {0x30, 0x1E8000},
    {0x30, 0x1F0000},
    {0x30, 0x1F8000},

};

/* Array of all the sector sizes */
static const WORD32 amd_29DL323D_bottom_sector_size_array[] = { 0x2000, 0x2000, 0x2000, 0x2000,
    0x2000, 0x2000, 0x2000, 0x2000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000
};
#endif


#if NAFLASH_WANT_TO_SUPPORT_M29W800AB
/*
 *
 *  Commands for STM29W800AB
 *
 *
 */

static const flash_cmd_t STM_M29W800AB_flash_id_enter_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x90, 0x5555}
};

static const flash_cmd_t STM_M29W800AB_flash_id_exit_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xF0, 0x5555}
};

static const flash_cmd_t STM_M29W800AB_flash_erase_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x80, 0x5555},
    {0xAA, 0x5555}, {0x55, 0x2AAA}
};

static const flash_cmd_t STM_M29W800AB_flash_write_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xA0, 0x5555}
};

/*
 * Array of commands user to erase sectors, starting with the first.
 */
static const flash_cmd_t STM_M29W800AB_flash_block_erase_cmd[19] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x02000},
    {0x30, 0x03000},
    {0x30, 0x04000},
    {0x30, 0x08000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000}
};

/* Array of all the sector sizes */

static const WORD32 STM_M29W800AB_flash_block_size_array[] = { 0x4000, 0x2000, 0x2000, 0x8000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000
};
#endif


#if NAFLASH_WANT_TO_SUPPORT_M29W160DB
/*
 *
 *  Commands for STM29W160DB
 *
 *
 */

static const flash_cmd_t STM_M29W160DB_flash_id_enter_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x90, 0x5555}
};

static const flash_cmd_t STM_M29W160DB_flash_id_exit_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xF0, 0x5555}
};

static const flash_cmd_t STM_M29W160DB_flash_erase_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x80, 0x5555},
    {0xAA, 0x5555}, {0x55, 0x2AAA}
};

static const flash_cmd_t STM_M29W160DB_flash_write_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xA0, 0x5555}
};

/*
 * Array of commands user to erase sectors, starting with the first.
 */
static const flash_cmd_t STM_M29W160DB_flash_block_erase_cmd[] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x02000},
    {0x30, 0x03000},
    {0x30, 0x04000},
    {0x30, 0x08000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000},
    {0x30, 0x80000},
    {0x30, 0x88000},
    {0x30, 0x90000},
    {0x30, 0x98000},
    {0x30, 0xA0000},
    {0x30, 0xA8000},
    {0x30, 0xB0000},
    {0x30, 0xB8000},
    {0x30, 0xC0000},
    {0x30, 0xC8000},
    {0x30, 0xD0000},
    {0x30, 0xD8000},
    {0x30, 0xE0000},
    {0x30, 0xE8000},
    {0x30, 0xF0000},
    {0x30, 0xF8000},
    {0x30, 0x100000},
    {0x30, 0x108000},
    {0x30, 0x110000},
    {0x30, 0x118000},
    {0x30, 0x120000},
    {0x30, 0x128000},
    {0x30, 0x130000},
    {0x30, 0x138000},
    {0x30, 0x140000},
    {0x30, 0x148000}
};

/* Array of all the sector sizes */

static const WORD32 STM_M29W160DB_flash_block_size_array[] = { 0x4000, 0x2000, 0x2000, 0x8000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000
};
#endif


#if NAFLASH_WANT_TO_SUPPORT_AM29LV160BB
/*
 * Command sequences for the AMD Am29LV160B
 */

/*
 * Array of commands user to erase sectors, starting with the first.
 */
#if BYTE_MODE
const flash_cmd_t AMD_29lv16_bottom_sector_erase_cmd[] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x02000}, {0x30, 0x03000}, {0x30, 0x4000}, {0x30, 0x08000},
    {0x30, 0x10000}, {0x30, 0x18000}, {0x30, 0x20000}, {0x30, 0x28000}, {0x30, 0x30000},
    {0x30, 0x38000}, {0x30, 0x40000}, {0x30, 0x48000}, {0x30, 0x50000}, {0x30, 0x58000},
    {0x30, 0x60000}, {0x30, 0x68000}, {0x30, 0x70000}, {0x30, 0x78000}, {0x30, 0x80000},
    {0x30, 0x88000}, {0x30, 0x90000}, {0x30, 0x98000}, {0x30, 0xA0000}, {0x30, 0xA8000},
    {0x30, 0xB0000}, {0x30, 0xB8000}, {0x30, 0xC0000}, {0x30, 0xC8000}, {0x30, 0xD0000},
    {0x30, 0xD8000}, {0x30, 0xE0000}, {0x30, 0xE8000}, {0x30, 0xF0000}, {0x30, 0xF8000}
};
#else

/* WORD mode (x16) */
const flash_cmd_t AMD_29lv16_bottom_sector_erase_cmd[] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x01000}, {0x30, 0x01800}, {0x30, 0x2000}, {0x30, 0x04000},
    {0x30, 0x8000}, {0x30, 0xC000}, {0x30, 0x10000}, {0x30, 0x14000}, {0x30, 0x18000},
    {0x30, 0x1C000}, {0x30, 0x20000}, {0x30, 0x24000}, {0x30, 0x28000}, {0x30, 0x2C000},
    {0x30, 0x30000}, {0x30, 0x34000}, {0x30, 0x38000}, {0x30, 0x3C000}, {0x30, 0x40000},
    {0x30, 0x44000}, {0x30, 0x48000}, {0x30, 0x4C000}, {0x30, 0x50000}, {0x30, 0x54000},
    {0x30, 0x58000}, {0x30, 0x5C000}, {0x30, 0x60000}, {0x30, 0x64000}, {0x30, 0x68000},
    {0x30, 0x6C000}, {0x30, 0x70000}, {0x30, 0x74000}, {0x30, 0x78000}, {0x30, 0x7C000}
};
#endif

/*
BYTE mode */
#if BYTE_MODE
const WORD32 amd_29lv16_sector_size_array[] = {
    0x4000, 0x2000, 0x2000, 0x8000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000
};
#else
static const WORD32 amd_29lv16_sector_size_array[] = {
    0x2000, 0x1000, 0x1000, 0x4000, 0x8000,
    0x8000, 0x8000, 0x8000, 0x8000, 0x8000,
    0x8000, 0x8000, 0x8000, 0x8000, 0x8000,
    0x8000, 0x8000, 0x8000, 0x8000, 0x8000,
    0x8000, 0x8000, 0x8000, 0x8000, 0x8000,
    0x8000, 0x8000, 0x8000, 0x8000, 0x8000,
    0x8000, 0x8000, 0x8000, 0x8000, 0x8000
};
#endif

/*
 * Command sequences for the AMD Am29v16
 */
static const flash_cmd_t AMD_29lv16_id_enter_cmd[] = {
#if BYTE_MODE
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x90, 0x5555}
#else
    {0xAA, 0xAAAA}, {0x55, 0x5554}, {0x90, 0xAAAA}
#endif
};

static const flash_cmd_t AMD_29lv16_id_exit_cmd[] = {
    {0xf0, 0x0000}
};

static const flash_cmd_t AMD_29lv16_write_cmd[] = {
#if BYTE_MODE
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xA0, 0x5555}
#else
    {0xAA, 0xAAAA}, {0x55, 0x5554}, {0xA0, 0xAAAA}
#endif
};

static const flash_cmd_t AMD_29lv16_erase_cmd[] = {
#if BYTE_MODE
    {0xAA, 0x5555},
    {0x55, 0x2AAA},
    {0x80, 0x5555},
    {0xAA, 0x5555},
    {0x55, 0x2AAA}
#else
    {0xAA, 0xAAAA},
    {0x55, 0x5554},
    {0x80, 0xAAAA},
    {0xAA, 0xAAAA},
    {0x55, 0x5554}
#endif
};
#endif




#if (NAFLASH_WANT_TO_SUPPORT_M29W320DB) || (NAFLASH_WANT_TO_SUPPORT_M29W320EB)
/*
 *
 *  Commands for STM29W320DB and STM29W320EB
 *
 *
 */

static const flash_cmd_t STM_M29W320xB_flash_id_enter_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x90, 0x5555}
};

static const flash_cmd_t STM_M29W320xB_flash_id_exit_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xF0, 0x5555}
};

static const flash_cmd_t STM_M29W320xB_flash_erase_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x80, 0x5555},
    {0xAA, 0x5555}, {0x55, 0x2AAA}
};

static const flash_cmd_t STM_M29W320xB_flash_write_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xA0, 0x5555}
};


#if NAFLASH_WANT_TO_SUPPORT_M29W320EB

/*
 * Array of commands user to erase sectors, starting with the first.
 */
const flash_cmd_t STM_M29W320EB_flash_block_erase_cmd[] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x01000},           
    {0x30, 0x02000},
    {0x30, 0x03000},
    {0x30, 0x04000},
    {0x30, 0x05000},
    {0x30, 0x06000},
    {0x30, 0x07000},
    {0x30, 0x8000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000},
    {0x30, 0x80000},
    {0x30, 0x88000},
    {0x30, 0x90000},
    {0x30, 0x98000},
    {0x30, 0xa0000},
    {0x30, 0xa8000},
    {0x30, 0xb0000},
    {0x30, 0xb8000},
    {0x30, 0xc0000},
    {0x30, 0xc8000},
    {0x30, 0xd0000},
    {0x30, 0xd8000},
    {0x30, 0xe0000},
    {0x30, 0xe8000},
    {0x30, 0xf0000},
    {0x30, 0xf8000},
    {0x30, 0x100000},
    {0x30, 0x108000},
    {0x30, 0x110000},
    {0x30, 0x118000},
    {0x30, 0x120000},
    {0x30, 0x128000},
    {0x30, 0x130000},
    {0x30, 0x138000},
    {0x30, 0x140000},
    {0x30, 0x148000},
    {0x30, 0x150000},
    {0x30, 0x158000},
    {0x30, 0x160000},
    {0x30, 0x168000},
    {0x30, 0x170000},
    {0x30, 0x178000},
    {0x30, 0x180000},
    {0x30, 0x188000},
    {0x30, 0x190000},
    {0x30, 0x198000},
    {0x30, 0x1a0000},
    {0x30, 0x1a8000},
    {0x30, 0x1b0000},
    {0x30, 0x1b8000},
    {0x30, 0x1c0000},
    {0x30, 0x1c8000},
    {0x30, 0x1d0000},
    {0x30, 0x1d8000},
    {0x30, 0x1e0000},
    {0x30, 0x1e8000},
    {0x30, 0x1f0000},
    {0x30, 0x1f8000}
};

/* Array of all the sector sizes */

const WORD32 STM_M29W320EB_flash_block_size_array[] = {
    0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000
};


#else

/*
 * Array of commands user to erase sectors, starting with the first.
 */
const flash_cmd_t STM_M29W320DB_flash_block_erase_cmd[] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x02000},
    {0x30, 0x03000},
    {0x30, 0x04000},
    {0x30, 0x08000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000},
    {0x30, 0x80000},
    {0x30, 0x88000},
    {0x30, 0x90000},
    {0x30, 0x98000},
    {0x30, 0xa0000},
    {0x30, 0xa8000},
    {0x30, 0xb0000},
    {0x30, 0xb8000},
    {0x30, 0xc0000},
    {0x30, 0xc8000},
    {0x30, 0xd0000},
    {0x30, 0xd8000},
    {0x30, 0xe0000},
    {0x30, 0xe8000},
    {0x30, 0xf0000},
    {0x30, 0xf8000},
    {0x30, 0x100000},
    {0x30, 0x108000},
    {0x30, 0x110000},
    {0x30, 0x118000},
    {0x30, 0x120000},
    {0x30, 0x128000},
    {0x30, 0x130000},
    {0x30, 0x138000},
    {0x30, 0x140000},
    {0x30, 0x148000},
    {0x30, 0x150000},
    {0x30, 0x158000},
    {0x30, 0x160000},
    {0x30, 0x168000},
    {0x30, 0x170000},
    {0x30, 0x178000},
    {0x30, 0x180000},
    {0x30, 0x188000},
    {0x30, 0x190000},
    {0x30, 0x198000},
    {0x30, 0x1a0000},
    {0x30, 0x1a8000},
    {0x30, 0x1b0000},
    {0x30, 0x1b8000},
    {0x30, 0x1c0000},
    {0x30, 0x1c8000},
    {0x30, 0x1d0000},
    {0x30, 0x1d8000},
    {0x30, 0x1e0000},
    {0x30, 0x1e8000},
    {0x30, 0x1f0000},
    {0x30, 0x1f8000}
};

/* Array of all the sector sizes */

const WORD32 STM_M29W320DB_flash_block_size_array[] = {
    0x4000, 0x2000, 0x2000, 0x8000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000
};

#endif

#endif



#if NAFLASH_WANT_TO_SUPPORT_M29DW323DB
/*
 *
 *  Commands for ST M29DW323DB
 *
 *
 */

static const flash_cmd_t STM_M29DW323DB_flash_id_enter_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x90, 0x5555}
};

static const flash_cmd_t STM_M29DW323DB_flash_id_exit_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xF0, 0x5555}
};

static const flash_cmd_t STM_M29DW323DB_flash_erase_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x80, 0x5555},
    {0xAA, 0x5555}, {0x55, 0x2AAA}
};

static const flash_cmd_t STM_M29DW323DB_flash_write_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xA0, 0x5555}
};

/*
 * Array of commands user to erase sectors, starting with the first.
 */
static const flash_cmd_t STM_M29DW323DB_flash_block_erase_cmd[] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x01000},
    {0x30, 0x02000},
    {0x30, 0x03000},
    {0x30, 0x04000},
    {0x30, 0x05000},
    {0x30, 0x06000},
    {0x30, 0x07000},
    {0x30, 0x08000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000},
    {0x30, 0x80000},
    {0x30, 0x88000},
    {0x30, 0x90000},
    {0x30, 0x98000},
    {0x30, 0xa0000},
    {0x30, 0xa8000},
    {0x30, 0xb0000},
    {0x30, 0xb8000},
    {0x30, 0xc0000},
    {0x30, 0xc8000},
    {0x30, 0xd0000},
    {0x30, 0xd8000},
    {0x30, 0xe0000},
    {0x30, 0xe8000},
    {0x30, 0xf0000},
    {0x30, 0xf8000},
    {0x30, 0x100000},
    {0x30, 0x108000},
    {0x30, 0x110000},
    {0x30, 0x118000},
    {0x30, 0x120000},
    {0x30, 0x128000},
    {0x30, 0x130000},
    {0x30, 0x138000},
    {0x30, 0x140000},
    {0x30, 0x148000},
    {0x30, 0x150000},
    {0x30, 0x158000},
    {0x30, 0x160000},
    {0x30, 0x168000},
    {0x30, 0x170000},
    {0x30, 0x178000},
    {0x30, 0x180000},
    {0x30, 0x188000},
    {0x30, 0x190000},
    {0x30, 0x198000},
    {0x30, 0x1a0000},
    {0x30, 0x1a8000},
    {0x30, 0x1b0000},
    {0x30, 0x1b8000},
    {0x30, 0x1c0000},
    {0x30, 0x1c8000},
    {0x30, 0x1d0000},
    {0x30, 0x1d8000},
    {0x30, 0x1e0000},
    {0x30, 0x1e8000},
    {0x30, 0x1f0000},
    {0x30, 0x1f8000}
};

/* Array of all the sector sizes */

static const WORD32 STM_M29DW323DB_flash_block_size_array[] = {
    0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000
};
#endif

#if NAFLASH_WANT_TO_SUPPORT_M29DW641F
/*
 *
 *  Commands for STM_29W641F
 *
 *
 */

static const flash_cmd_t STM_M29DW641F_flash_id_enter_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x90, 0x5555}
};

static const flash_cmd_t STM_M29DW641F_flash_id_exit_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xF0, 0x5555}
};

static const flash_cmd_t STM_M29DW641F_flash_erase_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x80, 0x5555},
    {0xAA, 0x5555}, {0x55, 0x2AAA}
};

static const flash_cmd_t STM_M29DW641F_flash_write_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xA0, 0x5555}
};

/*
 * Array of commands user to erase sectors, starting with the first.
 */
const flash_cmd_t STM_M29DW641F_flash_block_erase_cmd[] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x01000},
    {0x30, 0x02000},
    {0x30, 0x03000},
    {0x30, 0x04000},
    {0x30, 0x05000},
    {0x30, 0x06000},
    {0x30, 0x07000},
    {0x30, 0x08000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000},
    {0x30, 0x80000},
    {0x30, 0x88000},
    {0x30, 0x90000},
    {0x30, 0x98000},
    {0x30, 0xa0000},
    {0x30, 0xa8000},
    {0x30, 0xb0000},
    {0x30, 0xb8000},
    {0x30, 0xc0000},
    {0x30, 0xc8000},
    {0x30, 0xd0000},
    {0x30, 0xd8000},
    {0x30, 0xe0000},
    {0x30, 0xe8000},
    {0x30, 0xf0000},
    {0x30, 0xf8000},
    {0x30, 0x100000},
    {0x30, 0x108000},
    {0x30, 0x110000},
    {0x30, 0x118000},
    {0x30, 0x120000},
    {0x30, 0x128000},
    {0x30, 0x130000},
    {0x30, 0x138000},
    {0x30, 0x140000},
    {0x30, 0x148000},
    {0x30, 0x150000},
    {0x30, 0x158000},
    {0x30, 0x160000},
    {0x30, 0x168000},
    {0x30, 0x170000},
    {0x30, 0x178000},
    {0x30, 0x180000},
    {0x30, 0x188000},
    {0x30, 0x190000},
    {0x30, 0x198000},
    {0x30, 0x1a0000},
    {0x30, 0x1a8000},
    {0x30, 0x1b0000},
    {0x30, 0x1b8000},
    {0x30, 0x1c0000},
    {0x30, 0x1c8000},
    {0x30, 0x1d0000},
    {0x30, 0x1d8000},
    {0x30, 0x1e0000},
    {0x30, 0x1e8000},
    {0x30, 0x1f0000},
    {0x30, 0x1f8000},
    {0x30, 0x200000},
    {0x30, 0x208000},
    {0x30, 0x210000},
    {0x30, 0x218000},
    {0x30, 0x220000},
    {0x30, 0x228000},
    {0x30, 0x230000},
    {0x30, 0x238000},
    {0x30, 0x240000},
    {0x30, 0x248000},
    {0x30, 0x250000},
    {0x30, 0x258000},
    {0x30, 0x260000},
    {0x30, 0x268000},
    {0x30, 0x270000},
    {0x30, 0x278000},
    {0x30, 0x280000},
    {0x30, 0x288000},
    {0x30, 0x290000},
    {0x30, 0x298000},
    {0x30, 0x2a0000},
    {0x30, 0x2a8000},
    {0x30, 0x2b0000},
    {0x30, 0x2b8000},
    {0x30, 0x2c0000},
    {0x30, 0x2c8000},
    {0x30, 0x2d0000},
    {0x30, 0x2d8000},
    {0x30, 0x2e0000},
    {0x30, 0x2e8000},
    {0x30, 0x2f0000},
    {0x30, 0x2f8000},
    {0x30, 0x300000},
    {0x30, 0x308000},
    {0x30, 0x310000},
    {0x30, 0x318000},
    {0x30, 0x320000},
    {0x30, 0x328000},
    {0x30, 0x330000},
    {0x30, 0x338000},
    {0x30, 0x340000},
    {0x30, 0x348000},
    {0x30, 0x350000},
    {0x30, 0x358000},
    {0x30, 0x360000},
    {0x30, 0x368000},
    {0x30, 0x370000},
    {0x30, 0x378000},
    {0x30, 0x380000},
    {0x30, 0x388000},
    {0x30, 0x390000},
    {0x30, 0x398000},
    {0x30, 0x3a0000},
    {0x30, 0x3a8000},
    {0x30, 0x3b0000},
    {0x30, 0x3b8000},
    {0x30, 0x3c0000},
    {0x30, 0x3c8000},
    {0x30, 0x3d0000},
    {0x30, 0x3d8000},
    {0x30, 0x3e0000},
    {0x30, 0x3e8000},
    {0x30, 0x3f0000},
    {0x30, 0x3f8000},
    {0x30, 0x3f9000},
    {0x30, 0x3fa000},
    {0x30, 0x3fb000},
    {0x30, 0x3fc000},
    {0x30, 0x3fd000},
    {0x30, 0x3fe000},
    {0x30, 0x3ff000}
};

/* Array of all the sector sizes */

const WORD32 STM_M29DW641F_flash_block_size_array[] = {
    0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000
};
#endif

#if NAFLASH_WANT_TO_SUPPORT_S29AL032D || NAFLASH_WANT_TO_SUPPORT_EN29LV320B || NAFLASH_WANT_TO_SUPPORT_S29GL032N
/*
 *
 *  Commands for SPANSION_S29AL032D
 *
 *
 */

static const flash_cmd_t SPANSION_S29AL032D_flash_id_enter_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x90, 0x5555}
};

static const flash_cmd_t SPANSION_S29AL032D_flash_id_exit_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xF0, 0x5555}
};

static const flash_cmd_t SPANSION_S29AL032D_flash_erase_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x80, 0x5555},
    {0xAA, 0x5555}, {0x55, 0x2AAA}
};

static const flash_cmd_t SPANSION_S29AL032D_flash_write_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xA0, 0x5555}
};

/*
 * Array of commands user to erase sectors, starting with the first.
 */
const flash_cmd_t SPANSION_S29AL032D_flash_block_erase_cmd[] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x01000},
    {0x30, 0x02000},
    {0x30, 0x03000},
    {0x30, 0x04000},
    {0x30, 0x05000},
    {0x30, 0x06000},
    {0x30, 0x07000},
    {0x30, 0x08000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000},
    {0x30, 0x80000},
    {0x30, 0x88000},
    {0x30, 0x90000},
    {0x30, 0x98000},
    {0x30, 0xa0000},
    {0x30, 0xa8000},
    {0x30, 0xb0000},
    {0x30, 0xb8000},
    {0x30, 0xc0000},
    {0x30, 0xc8000},
    {0x30, 0xd0000},
    {0x30, 0xd8000},
    {0x30, 0xe0000},
    {0x30, 0xe8000},
    {0x30, 0xf0000},
    {0x30, 0xf8000},
    {0x30, 0x100000},
    {0x30, 0x108000},
    {0x30, 0x110000},
    {0x30, 0x118000},
    {0x30, 0x120000},
    {0x30, 0x128000},
    {0x30, 0x130000},
    {0x30, 0x138000},
    {0x30, 0x140000},
    {0x30, 0x148000},
    {0x30, 0x150000},
    {0x30, 0x158000},
    {0x30, 0x160000},
    {0x30, 0x168000},
    {0x30, 0x170000},
    {0x30, 0x178000},
    {0x30, 0x180000},
    {0x30, 0x188000},
    {0x30, 0x190000},
    {0x30, 0x198000},
    {0x30, 0x1a0000},
    {0x30, 0x1a8000},
    {0x30, 0x1b0000},
    {0x30, 0x1b8000},
    {0x30, 0x1c0000},
    {0x30, 0x1c8000},
    {0x30, 0x1d0000},
    {0x30, 0x1d8000},
    {0x30, 0x1e0000},
    {0x30, 0x1e8000},
    {0x30, 0x1f0000},
    {0x30, 0x1f8000}
};

/* Array of all the sector sizes */

const WORD32 SPANSION_S29AL032D_flash_block_size_array[] = {
    0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 0x2000, 
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000,
    0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000
};
#endif


#if NAFLASH_WANT_TO_SUPPORT_S29GL064N
/*
 *
 *  Commands for Spansion S29GL128P
 *
 *
 */

static const flash_cmd_t SPANSION_S29GL064N_flash_id_enter_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x90, 0x5555}
};

static const flash_cmd_t SPANSION_S29GL064N_flash_id_exit_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xF0, 0x5555}
};

static const flash_cmd_t SPANSION_S29GL064N_flash_erase_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0x80, 0x5555},
    {0xAA, 0x5555}, {0x55, 0x2AAA}
};

static const flash_cmd_t SPANSION_S29GL064N_flash_write_cmd[] = {
    {0xAA, 0x5555}, {0x55, 0x2AAA}, {0xA0, 0x5555}
};

/*
 * Array of commands user to erase sectors, starting with the first.
 */
const flash_cmd_t SPANSION_S29GL064N_flash_block_erase_cmd[] = {
    {0x30, 0x00000},            /* Command to erase the first sector */
    {0x30, 0x08000},
    {0x30, 0x10000},
    {0x30, 0x18000},
    {0x30, 0x20000},
    {0x30, 0x28000},
    {0x30, 0x30000},
    {0x30, 0x38000},
    {0x30, 0x40000},
    {0x30, 0x48000},
    {0x30, 0x50000},
    {0x30, 0x58000},
    {0x30, 0x60000},
    {0x30, 0x68000},
    {0x30, 0x70000},
    {0x30, 0x78000},
    {0x30, 0x80000},
    {0x30, 0x88000},
    {0x30, 0x90000},
    {0x30, 0x98000},
    {0x30, 0xA0000},
    {0x30, 0xA8000},
    {0x30, 0xB0000},
    {0x30, 0xB8000},
    {0x30, 0xC0000},
    {0x30, 0xC8000},
    {0x30, 0xD0000},
    {0x30, 0xD8000},
    {0x30, 0xE0000},
    {0x30, 0xE8000},
    {0x30, 0xF0000},
    {0x30, 0xF8000},
    {0x30, 0x100000},
    {0x30, 0x108000},
    {0x30, 0x110000},
    {0x30, 0x118000},
    {0x30, 0x120000},
    {0x30, 0x128000},
    {0x30, 0x130000},
    {0x30, 0x138000},
    {0x30, 0x140000},
    {0x30, 0x148000},
    {0x30, 0x150000},
    {0x30, 0x158000},
    {0x30, 0x160000},
    {0x30, 0x168000},
    {0x30, 0x170000},
    {0x30, 0x178000},
    {0x30, 0x180000},
    {0x30, 0x188000},
    {0x30, 0x190000},
    {0x30, 0x198000},
    {0x30, 0x1A0000},
    {0x30, 0x1A8000},
    {0x30, 0x1B0000},
    {0x30, 0x1B8000},
    {0x30, 0x1C0000},
    {0x30, 0x1C8000},
    {0x30, 0x1D0000},
    {0x30, 0x1D8000},
    {0x30, 0x1E0000},
    {0x30, 0x1E8000},
    {0x30, 0x1F0000},
    {0x30, 0x1F8000},
    {0x30, 0x200000},
    {0x30, 0x208000},
    {0x30, 0x210000},
    {0x30, 0x218000},
    {0x30, 0x220000},
    {0x30, 0x228000},
    {0x30, 0x230000},
    {0x30, 0x238000},
    {0x30, 0x240000},
    {0x30, 0x248000},
    {0x30, 0x250000},
    {0x30, 0x258000},
    {0x30, 0x260000},
    {0x30, 0x268000},
    {0x30, 0x270000},
    {0x30, 0x278000},
    {0x30, 0x280000},
    {0x30, 0x288000},
    {0x30, 0x290000},
    {0x30, 0x298000},
    {0x30, 0x2A0000},
    {0x30, 0x2A8000},
    {0x30, 0x2B0000},
    {0x30, 0x2B8000},
    {0x30, 0x2C0000},
    {0x30, 0x2C8000},
    {0x30, 0x2D0000},
    {0x30, 0x2D8000},
    {0x30, 0x2E0000},
    {0x30, 0x2E8000},
    {0x30, 0x2F0000},
    {0x30, 0x2F8000},
    {0x30, 0x300000},
    {0x30, 0x308000},
    {0x30, 0x310000},
    {0x30, 0x318000},
    {0x30, 0x320000},
    {0x30, 0x328000},
    {0x30, 0x330000},
    {0x30, 0x338000},
    {0x30, 0x340000},
    {0x30, 0x348000},
    {0x30, 0x350000},
    {0x30, 0x358000},
    {0x30, 0x360000},
    {0x30, 0x368000},
    {0x30, 0x370000},
    {0x30, 0x378000},
    {0x30, 0x380000},
    {0x30, 0x388000},
    {0x30, 0x390000},
    {0x30, 0x398000},
    {0x30, 0x3A0000},
    {0x30, 0x3A8000},
    {0x30, 0x3B0000},
    {0x30, 0x3B8000},
    {0x30, 0x3C0000},
    {0x30, 0x3C8000},
    {0x30, 0x3D0000},
    {0x30, 0x3D8000},
    {0x30, 0x3E0000},
    {0x30, 0x3E8000},
    {0x30, 0x3F0000},
    {0x30, 0x3F8000}
};

/* Spansion S29GL064N has uniform sized sectors  */
/* so no need for an array of flash sector sizes */

#endif


static const flash_id_table_t flash_id_table[] = {
    /* Insert Flashes used by Forth Systeme */
    /* AMD AM29LV641 */
#if NAFLASH_WANT_TO_SUPPORT_AM29LV641
    {
     0x01, 0x00, AM29LV641_ID, 0x01, 0, 0, 0, 0, AM29LV641_SECTORS, AM29LV641_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29lv641_sector_erase_cmd,
     (WORD32 *) 0}
    ,
#endif
#if (NAFLASH_WANT_TO_SUPPORT_MBM29LV650U) || (NAFLASH_WANT_TO_SUPPORT_MX29LV640BUTI90)
    /* Fujitsu MBM29LV650U & Macronix MX29LV640BUTI90 (AMD AM29LV641 Compatible) */
    {
     0x04, 0x00, AM29LV641_ID, 0x01, 0, 0, 0, 0, AM29LV641_SECTORS, AM29LV641_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29lv641_sector_erase_cmd,
     (WORD32 *) 0}
    ,
#endif
#if (NAFLASH_WANT_TO_SUPPORT_M29W641DL) || (NAFLASH_WANT_TO_SUPPORT_M29W641DH)
    /* ST Microelectronic M29W641DL and M29W641DH (AMD AM29LV641 Compatible) */
    {
     0x20, 0x00, M29W641D_ID, 0x01, 0, 0, 0, 0, AM29LV641_SECTORS, AM29LV641_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29lv641_sector_erase_cmd,
     (WORD32 *) 0}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AM29LV641MH
    /*AMD MirrorBit Tech AM29LV641MH/L (AMD AM29LV641 Compatible) */
    {
     0x01, 0x00, AM29LV641M_ID1, 0x01, AM29LV641M_ID2, 0x0E, 0, 0, AM29LV641_SECTORS, AM29LV641_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE,
     120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29lv641_sector_erase_cmd,
     (WORD32 *) 0}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AM29LV128M
    /*AMD MirrorBit Tech AM29LV128M */
    {
     0x01, 0x00, AM29LV128M_ID1, 0x01, AM29LV128M_ID2, 0x0E, 0, 0, AM29LV128M_SECTORS, AM29LV128M_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE,
     120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29lv128M_sector_erase_cmd,
     (WORD32 *) 0}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AM29LV160T
    /* AMD AM29LV160 Top Type */
    {
     0x01, 0x00, AM29LV160T_ID, 0x01, 0, 0, 0, 0, AM29LV160_SECTORS, VARIABLE_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29lv160_top_sector_erase_cmd,
     (WORD32 *) amd_29lv160_top_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AM29LV160B
    /* AMD AM29LV160 Bottom Type */
    {
     0x01, 0x00, AM29LV160B_ID, 0x01, 0, 0, 0, 0, AM29LV160_SECTORS, VARIABLE_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29lv160_bottom_sector_erase_cmd,
     (WORD32 *) amd_29lv160_bottom_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_MBM29LV160T
    /* FUJITSU MBM 29LV160 Top Type */
    {
     0x04, 0x00, AM29LV160T_ID, 0x01, 0, 0, 0, 0, AM29LV160_SECTORS, VARIABLE_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29lv160_top_sector_erase_cmd,
     (WORD32 *) amd_29lv160_top_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_MBM29LV160B
    /* FUJITSU MBM 29LV160 Bottom Type */
    {
     0x04, 0x00, AM29LV160B_ID, 0x01, 0, 0, 0, 0, AM29LV160_SECTORS, VARIABLE_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29lv160_bottom_sector_erase_cmd,
     (WORD32 *) amd_29lv160_bottom_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_M29W160T
    /* ST Microelectronics M29W160 Top Type */
    /* !! Attention Not tested use to your own risc
       {
       0x20, 0x00, AM29LV160T_ID, 0x01,0,0, AM29LV160_SECTORS, VARIABLE_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
       (flash_cmd_t *) AMD_flash_id_enter_cmd,      sizeof( AMD_flash_id_enter_cmd ) / sizeof( flash_cmd_t ),
       (flash_cmd_t *) AMD_flash_id_exit_cmd,       sizeof( AMD_flash_id_exit_cmd ) / sizeof( flash_cmd_t ),
       (flash_cmd_t *) AMD_flash_erase_cmd, sizeof(AMD_flash_erase_cmd) / sizeof(flash_cmd_t),
       (flash_cmd_t *) AMD_flash_write_cmd, sizeof(AMD_flash_write_cmd) / sizeof(flash_cmd_t),
       (flash_cmd_t *)AMD_29lv160_top_sector_erase_cmd,
       (WORD32 *)amd_29lv160_top_sector_size_array
       }, */
#endif
#if NAFLASH_WANT_TO_SUPPORT_M29W160B
    /* ST Microelectronics M29W160 Bottom Type */
    /*!! Attention Not tested use to your own risc
       {
       0x20, 0x00, AM29LV160B_ID, 0x01,0,0, AM29LV160_SECTORS, VARIABLE_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
       (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof( AMD_flash_id_enter_cmd ) / sizeof( flash_cmd_t ),
       (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof( AMD_flash_id_exit_cmd ) / sizeof( flash_cmd_t ),
       (flash_cmd_t *) AMD_flash_erase_cmd, sizeof(AMD_flash_erase_cmd) / sizeof(flash_cmd_t),
       (flash_cmd_t *) AMD_flash_write_cmd, sizeof(AMD_flash_write_cmd) / sizeof(flash_cmd_t),
       (flash_cmd_t *)AMD_29lv160_bottom_sector_erase_cmd,
       (WORD32 *)amd_29lv160_bottom_sector_size_array
       }, */
#endif
#if NAFLASH_WANT_TO_SUPPORT_LH28F800SG
    {
     0xB0, 0x00, 0x50, 0x01, 0, 0, 0, 0, SHARP_FLASH_SECTORS, SHARP_SECTOR_SIZE, SHARP_PROG_SECTOR_SIZE, 70,
     (flash_cmd_t *) SHARP_flash_id_enter_cmd, sizeof (SHARP_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SHARP_flash_id_exit_cmd, sizeof (SHARP_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SHARP_flash_erase_cmd, sizeof (SHARP_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SHARP_flash_write_cmd, sizeof (SHARP_flash_write_cmd) / sizeof (flash_cmd_t),
     NULL,
     (WORD32 *) 0}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AT29C040
    {
     0x1F, 0x00, 0x5B, 0x01, 0, 0, 0, 0, AT_FLASH_SECTORS, AT_SECTOR_SIZE, AT_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AT_flash_id_enter_cmd, sizeof (AT_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     0, 0,
     0, 0,
     (flash_cmd_t *) AT_flash_write_cmd, sizeof (AT_flash_write_cmd) / sizeof (flash_cmd_t),
     NULL,
     (WORD32 *) 0}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AT29C040A
    {
     0x1F, 0x00, 0xA4, 0x01, 0, 0, 0, 0, AT_A_FLASH_SECTORS, AT_A_SECTOR_SIZE, AT_A_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AT_flash_id_enter_cmd, sizeof (AT_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AT_flash_id_exit_cmd, sizeof (AT_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     0, 0,
     (flash_cmd_t *) AT_flash_write_cmd, sizeof (AT_flash_write_cmd) / sizeof (flash_cmd_t),
     NULL,
     (WORD32 *) 0}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_MX28F4000
    {
     0xC2, 0x00, 0x41, 0x01, 0, 0, 0, 0, MX_FLASH_SECTORS, MX_SECTOR_SIZE, MX_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) MX_flash_id_cmd, sizeof (MX_flash_id_cmd) / sizeof (flash_cmd_t),
     0, 0,
     (flash_cmd_t *) MX_flash_erase_cmd, sizeof (MX_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) MX_flash_write_cmd, sizeof (MX_flash_write_cmd) / sizeof (flash_cmd_t),
     NULL,
     (WORD32 *) 0}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_SST28SF040
    {
     0xBF, 0x00, 0xA4, 0x01, 0, 0, 0, 0, SST_FLASH_SECTORS, SST_SECTOR_SIZE, SST_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) SST_flash_id_enter_cmd, sizeof (SST_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SST_flash_id_exit_cmd, sizeof (SST_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SST_flash_erase_cmd, sizeof (SST_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SST_flash_write_cmd, sizeof (SST_flash_write_cmd) / sizeof (flash_cmd_t),
     NULL,
     (WORD32 *) 0}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AM29F800BB
    {
     0x01, 0x00, 0x2258, 0x01, 0, 0, 0, 0, AMD_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_bottom_sector_erase_cmd,
     (WORD32 *) amd_bottom_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AM29LV800BB
    {
     0x01, 0x00, 0x225b, 0x01, 0, 0, 0, 0, AMD_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_bottom_sector_erase_cmd,
     (WORD32 *) amd_bottom_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AM29F800BT
    {
     0x01, 0x00, 0x22d6, 0x01, 0, 0, 0, 0, AMD_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_top_sector_erase_cmd,
     (WORD32 *) amd_top_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AM29LV800BT
    {
     0x01, 0x00, 0x22da, 0x01, 0, 0, 0, 0, AMD_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_top_sector_erase_cmd,
     (WORD32 *) amd_top_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_MBM29LV800BE
    {
     0x04, 0x00, 0x225b, 0x01, 0, 0, 0, 0, FUJITSU_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, FUJITSU_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_bottom_sector_erase_cmd,
     (WORD32 *) amd_bottom_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AT49BV8011
    {
     0x1F, 0x00, 0x00CB, 0x01, 0, 0, 0, 0, AT_49BV8011_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, AT_49BV8011_PROG_SECTOR_SIZE, 160,
     (flash_cmd_t *) AT_flash_id_enter_cmd, sizeof (AT_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AT_flash_id_exit_cmd, sizeof (AT_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AT_flash_erase_cmd, sizeof (AT_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AT_flash_write_cmd, sizeof (AT_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AT_flash_AT49BV8011_sector_erase_cmd,
     (WORD32 *) AT_AT49BV8011_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AT49BV8011T
    {
     0x1F, 0x00, 0x004A, 0x01, 0, 0, 0, 0, AT_49BV8011_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, AT_49BV8011_PROG_SECTOR_SIZE, 160,
     (flash_cmd_t *) AT_flash_id_enter_cmd, sizeof (AT_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AT_flash_id_exit_cmd, sizeof (AT_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AT_flash_erase_cmd, sizeof (AT_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AT_flash_write_cmd, sizeof (AT_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AT_flash_AT49BV8011T_sector_erase_cmd,
     (WORD32 *) AT_AT49BV8011T_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_SST39VF800
    {
     0xBF, 0x00, 0x2781, 0x01, 0, 0, 0, 0, SST_39VF800_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, SST_39VF800_PROG_SECTOR_SIZE, 160,
     (flash_cmd_t *) SST_39VF800_flash_id_enter_cmd, sizeof (SST_39VF800_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SST_39VF800_flash_id_exit_cmd, sizeof (SST_39VF800_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SST_39VF800_flash_erase_cmd, sizeof (SST_39VF800_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SST_39VF800_flash_write_cmd, sizeof (SST_39VF800_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SST_39VF800_flash_block_erase_cmd,
     (WORD32 *) SST_39VF800_flash_block_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_M29W800AB
    {
     0x20, 0x00, 0x005B, 0x01, 0, 0, 0, 0, STM_M29W800AB_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, STM_M29W800AB_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) STM_M29W800AB_flash_id_enter_cmd, sizeof (STM_M29W800AB_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W800AB_flash_id_exit_cmd, sizeof (STM_M29W800AB_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W800AB_flash_erase_cmd, sizeof (STM_M29W800AB_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W800AB_flash_write_cmd, sizeof (STM_M29W800AB_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W800AB_flash_block_erase_cmd,
     (WORD32 *) STM_M29W800AB_flash_block_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AM29DL323DT
    {
     0x01, 0x00, 0x2250, 0x01, 0, 0, 0, 0, AMD_29DL323D_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, AMD_29DL323D_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29DL323D_flash_top_sector_erase_cmd,
     (WORD32 *) amd_29DL323D_top_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AM29DL323DTB
    {
     0x01, 0x00, 0x2253, 0x01, 0, 0, 0, 0, AMD_29DL323D_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, AMD_29DL323D_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29DL323D_flash_bottom_sector_erase_cmd,
     (WORD32 *) amd_29DL323D_bottom_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AM29LV160BB
    /* asbury flash id 0x12 */
    {
     0x01, 0x00, 0x2249, 0x01, 0, 0, 0, 0, 35 /* AMD_FLASH_SECTORS */ , VARIABLE_SECTOR_SIZE, AMD_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_29lv16_id_enter_cmd, sizeof (AMD_29lv16_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29lv16_id_exit_cmd, sizeof (AMD_29lv16_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29lv16_erase_cmd, sizeof (AMD_29lv16_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29lv16_write_cmd, sizeof (AMD_29lv16_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29lv16_bottom_sector_erase_cmd,
     (WORD32 *) amd_29lv16_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AT49BV1614A
    {
     0x1F, 0x00, 0x00C0, 0x01, 0, 0, 0, 0, AT_49BV1614A_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, AT_49BV1614A_PROG_SECTOR_SIZE, 160,
     (flash_cmd_t *) AT_flash_id_enter_cmd, sizeof (AT_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AT_flash_id_exit_cmd, sizeof (AT_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AT_flash_erase_cmd, sizeof (AT_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AT_flash_write_cmd, sizeof (AT_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AT_flash_AT49BV1614A_sector_erase_cmd,
     (WORD32 *) AT_AT49BV1614A_sector_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_M29W320DB
    {
     0x20, 0x00, 0x22cb, 0x01, 0, 0, 0, 0, STM_M29W320DB_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, STM_M29W320xB_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) STM_M29W320xB_flash_id_enter_cmd, sizeof (STM_M29W320xB_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W320xB_flash_id_exit_cmd, sizeof (STM_M29W320xB_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W320xB_flash_erase_cmd, sizeof (STM_M29W320xB_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W320xB_flash_write_cmd, sizeof (STM_M29W320xB_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W320DB_flash_block_erase_cmd,
     (WORD32 *) STM_M29W320DB_flash_block_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_M29W320EB
    {
     0x20, 0x00, 0x2257, 0x01, 0, 0, 0, 0, STM_M29W320EB_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, STM_M29W320xB_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) STM_M29W320xB_flash_id_enter_cmd, sizeof (STM_M29W320xB_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W320xB_flash_id_exit_cmd, sizeof (STM_M29W320xB_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W320xB_flash_erase_cmd, sizeof (STM_M29W320xB_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W320xB_flash_write_cmd, sizeof (STM_M29W320xB_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W320EB_flash_block_erase_cmd,
     (WORD32 *) STM_M29W320EB_flash_block_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_M29DW323DB
    {
     0x20, 0x00, 0x225f, 0x01, 0, 0, 0, 0, STM_M29DW323DB_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, STM_M29DW323DB_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) STM_M29DW323DB_flash_id_enter_cmd, sizeof (STM_M29DW323DB_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29DW323DB_flash_id_exit_cmd, sizeof (STM_M29DW323DB_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29DW323DB_flash_erase_cmd, sizeof (STM_M29DW323DB_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29DW323DB_flash_write_cmd, sizeof (STM_M29DW323DB_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29DW323DB_flash_block_erase_cmd,
     (WORD32 *) STM_M29DW323DB_flash_block_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_M29W160DB
    {
     0x20, 0x00, 0x2249, 0x01, 0, 0, 0, 0, STM_M29W160DB_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, STM_M29W160DB_PROG_SECTOR_SIZE, 90,
     (flash_cmd_t *) STM_M29W160DB_flash_id_enter_cmd, sizeof (STM_M29W160DB_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W160DB_flash_id_exit_cmd, sizeof (STM_M29W160DB_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W160DB_flash_erase_cmd, sizeof (STM_M29W160DB_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W160DB_flash_write_cmd, sizeof (STM_M29W160DB_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29W160DB_flash_block_erase_cmd,
     (WORD32 *) STM_M29W160DB_flash_block_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_M29DW641F
    {
     0x20, 0x00, 0x227e, 0x01, 0x2203, 0x0E, 0x2200, 0x0F, STM_M29DW641F_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, STM_M29DW641F_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) STM_M29DW641F_flash_id_enter_cmd, sizeof (STM_M29DW641F_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29DW641F_flash_id_exit_cmd, sizeof (STM_M29DW641F_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29DW641F_flash_erase_cmd, sizeof (STM_M29DW641F_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29DW641F_flash_write_cmd, sizeof (STM_M29DW641F_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) STM_M29DW641F_flash_block_erase_cmd,
     (WORD32 *) STM_M29DW641F_flash_block_size_array}
    ,
#endif
#if NAFLASH_WANT_TO_SUPPORT_AM29LV320DB
    {
     0x01, 0x00, 0x22f9, 0x01, 0, 0, 0, 0, AMD_29DL323D_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, AMD_29DL323D_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) AMD_flash_id_enter_cmd, sizeof (AMD_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_id_exit_cmd, sizeof (AMD_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_erase_cmd, sizeof (AMD_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_flash_write_cmd, sizeof (AMD_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) AMD_29DL323D_flash_bottom_sector_erase_cmd,
     (WORD32 *) amd_29DL323D_bottom_sector_size_array}
    ,
#endif

#if NAFLASH_WANT_TO_SUPPORT_S29AL032D
    {
     0x01, 0x00, SPANSION_S29AL032D_ID1, 0x01, 0, 0, 0, 0, SPANSION_S29AL032D_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, SPANSION_S29AL032D_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) SPANSION_S29AL032D_flash_id_enter_cmd, sizeof (SPANSION_S29AL032D_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29AL032D_flash_id_exit_cmd, sizeof (SPANSION_S29AL032D_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29AL032D_flash_erase_cmd, sizeof (SPANSION_S29AL032D_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29AL032D_flash_write_cmd, sizeof (SPANSION_S29AL032D_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29AL032D_flash_block_erase_cmd,
     (WORD32 *) SPANSION_S29AL032D_flash_block_size_array}
    ,
#endif     

#if NAFLASH_WANT_TO_SUPPORT_EN29LV320B
    {
     0x7f, 0x00, EON_EN29LV320B_ID1, 0x01, 0, 0, 0, 0, EON_EN29LV320B_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, EON_EN29LV320B_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) SPANSION_S29AL032D_flash_id_enter_cmd, sizeof (SPANSION_S29AL032D_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29AL032D_flash_id_exit_cmd, sizeof (SPANSION_S29AL032D_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29AL032D_flash_erase_cmd, sizeof (SPANSION_S29AL032D_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29AL032D_flash_write_cmd, sizeof (SPANSION_S29AL032D_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29AL032D_flash_block_erase_cmd,
     (WORD32 *) SPANSION_S29AL032D_flash_block_size_array}
    ,
#endif

#if NAFLASH_WANT_TO_SUPPORT_S29GL032N
    {
     0x01, 0x00, SPANSION_S29GL032N_ID1, 0x01, 0x221a, 0x0e, 0x2200, 0x0f, SPANSION_S29GL032N_FLASH_SECTORS, VARIABLE_SECTOR_SIZE, SPANSION_S29GL032N_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) SPANSION_S29AL032D_flash_id_enter_cmd, sizeof (SPANSION_S29AL032D_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29AL032D_flash_id_exit_cmd, sizeof (SPANSION_S29AL032D_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29AL032D_flash_erase_cmd, sizeof (SPANSION_S29AL032D_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29AL032D_flash_write_cmd, sizeof (SPANSION_S29AL032D_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29AL032D_flash_block_erase_cmd,
     (WORD32 *) SPANSION_S29AL032D_flash_block_size_array}
    ,
#endif

#if NAFLASH_WANT_TO_SUPPORT_S29GL064N
    {
     0x01, 0x00, SPANSION_S29GL064N_ID1, 0x01, SPANSION_S29GL064N_ID2, 0x0E, SPANSION_S29GL064N_ID3, 0x0F, SPANSION_S29GL064N_FLASH_SECTORS, SPANSION_S29GL064N_FLASH_SECTOR_SIZE, SPANSION_S29GL064N_PROG_SECTOR_SIZE, 120,
     (flash_cmd_t *) SPANSION_S29GL064N_flash_id_enter_cmd, sizeof (SPANSION_S29GL064N_flash_id_enter_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29GL064N_flash_id_exit_cmd, sizeof (SPANSION_S29GL064N_flash_id_exit_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29GL064N_flash_erase_cmd, sizeof (SPANSION_S29GL064N_flash_erase_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29GL064N_flash_write_cmd, sizeof (SPANSION_S29GL064N_flash_write_cmd) / sizeof (flash_cmd_t),
     (flash_cmd_t *) SPANSION_S29GL064N_flash_block_erase_cmd,
     (WORD32 *) 0}
    ,
#endif
};


const WORD8 naflash_num_id = sizeof (flash_id_table) / sizeof (flash_id_table_t);
const flash_id_table_t *naflash_id_table = flash_id_table;

